'From VisualWorks® NonCommercial, Release 5i.4 of August 9, 2001 on March 29, 2002 at 2:05:38 pm'!


Smalltalk.Core defineClass: #Locale
	superclass: #{Core.Object}
	indexedType: #none
	private: false
	instanceVariableNames: 'name collationPolicy currencyPolicy numberPolicy timePolicy compoundTextEncodeOrder cutPasteEncoding defaultStreamEncoder ignoreSerifEncodings needsInputMethod preferredEncodings preferredFontFamily preferredPixelSize defaultPaperSize '
	classInstanceVariableNames: 'dependents '
	imports: ''
	category: 'Globalization'!

Core.Locale defineSharedVariable: #LocaleDictionary
	private: false
	constant: false
	category: 'As yet unclassified'
	initializer: nil!

Core.Locale defineSharedVariable: #LocaleMap
	private: false
	constant: false
	category: 'As yet unclassified'
	initializer: nil!

Core.Locale defineSharedVariable: #CurrentLocale
	private: false
	constant: false
	category: 'As yet unclassified'
	initializer: nil!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!


!Core.Locale class methodsFor: 'initialization'!

initialize
"Initialise la locale a l'ouverture de VisualWorks en installant  
nos locales francaise et anglaise. La locale activee par defaut est l'americaine
hm 99"

	ObjectMemory removeDependent: self.
	ObjectMemory addDependent: self.
	LocaleMap == nil ifTrue: [LocaleMap := Dictionary new].
	LocaleDictionary == nil ifTrue: [LocaleDictionary := Dictionary new].
	(LocaleDictionary includesKey: #unix)
		ifFalse: [LocaleDictionary at: #unix put: IdentityDictionary new].
	(LocaleDictionary includesKey: #mac)
		ifFalse: [LocaleDictionary at: #mac put: IdentityDictionary new].
	(LocaleDictionary includesKey: #win32)
		ifFalse: [LocaleDictionary at: #win32 put: IdentityDictionary new].
	(LocaleDictionary includesKey: #os2)
		ifFalse: [LocaleDictionary at: #os2 put: IdentityDictionary new].
	self installCLocale.
	self installFrLocale.
	self installEnLocale.
	CurrentLocale := self getLocale: #C.
	^self! !

!Core.Locale class methodsFor: 'accessing'!

availableLocales
	"Return collection of available locale names for this platform."
	"Locale availableLocales"

	^(LocaleDictionary at: self platformName) keys!

current
	"Return current locale instance"
	"Locale current"

	^CurrentLocale!

listLocaleNamesFor: stLocaleSymbol
	"Unmap Smalltalk locale symbol to local name(s) of the current operating system."

	| os list |
	list := List new.
	os := ExternalInterface currentPlatform at: 2.
	self localeMap keysAndValuesDo: [ :key :val |
		(val == stLocaleSymbol and: [(key at: 1) match: os])
			ifTrue: [list add: (key at: 2)]].
	^list!

localeMap
	"Answer the LocaleMap"

	^LocaleMap!

mapLocaleName: osLocaleSymbol
	"Map locale name on the current operating system to Smalltalk locale symbol."

	| os |
	os := OSHandle currentPlatformID.
	self localeMap keysAndValuesDo: [ :key :val |
		(((key at: 1) match: os) and: [(key at: 2) == osLocaleSymbol])
			ifTrue: [^val]].
	^osLocaleSymbol!

named: aLocaleName

	^self getLocale: aLocaleName!

preferredLocaleName
	"Locale preferredLocaleName."

	^ByteString fromBytes: self getLocaleName!

set: aSymbol
	"Set the current locale to aSymbol's locale.
	 'Locale set: #C'
	 'Locale set: #ja_JP.EUC'"	

	| names localeSet |
	names := self listLocaleNamesFor: aSymbol.
	names do:
		[:nam |
		localeSet := Object errorSignal
			handle: [:ex | ex returnWith: false]
			do: [InputManager default setLocale: nam.
				true].
		localeSet
			ifTrue:
				[self internalSet: aSymbol.
				^self]].
	localeSet := Object errorSignal
		handle: [:ex | ex returnWith: false]
		do: [InputManager default setLocale: aSymbol.
			true].
	localeSet
		ifTrue:
			[self internalSet: aSymbol.
			^self].
	Dialog warn: ('Locale "<1s>" not supported by the host.<n>C libraries will use "<2s>" instead'
			expandMacrosWith: aSymbol
			with: Locale preferredLocaleName).!

setDefault
	"Set the locale according to external environment."
	"Locale setDefault"

	| locale |
	locale := self mapLocaleName:
				self preferredLocaleName asSymbol.
	self internalSet: locale! !

!Core.Locale class methodsFor: 'private'!

addLocale: aLocale platform: platform
	"Install a new locale."

	| localeName |

	localeName := aLocale name.
	(LocaleDictionary at: platform) at: localeName put: aLocale!

getLocale: localeName
	^(LocaleDictionary at: self platformName) 
		at: localeName
		ifAbsent: [nil]!

getLocaleName
	"Locale preferredLocaleName."

	<primitive: 471>
	^IOAccessor convertSimpleStringForPlatform: 'C'!

internalSet: aSymbol
	"Set the current locale to aSymbol's locale."

	CurrentLocale := self getLocale: aSymbol.

	CurrentLocale == nil ifTrue: [
		CurrentLocale := self getLocale: #C.
		Dialog warn: 'Could not find locale: ', aSymbol, '.  Using C locale instead.'].

	"update dependents that there is a new locale (not font dependents)"
	ByteArray purgeCaches.
	self changed: #locale_early with: false.
	self changed: #locale with: false.

	"Next line needed under JWindows because:
		a) default font must use appropriate encoding
		b) font names with international characters must be correctly converted to Unicode"
	Screen default installFontPolicy.

	"update font dependents that fonts may have been updated"
	self changed: #localeFonts with: false.
	TextAttributes resetViews.
	InputManager setDefault!

myDependents
	^dependents!

myDependents: dependentsOrNil
	dependents := dependentsOrNil!

update: anAspect with: arguments from: anObject
	(anObject == ObjectMemory and: [anAspect == #earlySystemInstallation])
		ifTrue: [self platformHookup].! !

!Core.Locale class methodsFor: 'start-up'!

initialHookup
	"Set-up platform specific state."

	| locale |
	locale := self mapLocaleName:
				self preferredLocaleName asSymbol.

	CurrentLocale := self getLocale: locale.
	CurrentLocale == nil ifTrue: [
		CurrentLocale := self getLocale: #C.].
	ByteArray purgeCaches.
	self changed: #locale_early with: CurrentLocale!

platformHookup
	"Set-up platform specific state."

	InputManager platformHookup.

	"update dependents that there is a new locale (not font dependents)"
	self changed: #locale with: true.

	self changed: #localeFonts with: true.
	InputManager setDefault!

platformName
	"Return one of (#unix #mac #os2 #win32)"

	^OSHandle currentOS! !

!Core.Locale class methodsFor: 'installation'!

install
	"For the moment, do nothing, since Locale class > initialize is
	already installing the C locale."

	^self!

installCLocale
	| locale copy |

	locale := self new.
	locale
		name: #C;
		collationPolicy: nil;
		compoundTextEncodeOrder: #(#ascii #iso1);
		currencyPolicy: nil;
		numberPolicy: nil;
		timePolicy: nil;
		cutPasteEncoding: #CompoundText;
		defaultPaperSize: 8.5 @ 11.0; "Letter paper size in inch"
		defaultStreamEncoder: ISO8859L1String encoder;
		ignoreSerifEncodings: #();
		needsInputMethod: false;
		preferredEncodings: nil;
		preferredFontFamily: #();
		preferredPixelSize: 12.

	copy := locale copy.
	self addLocale: copy platform: #unix.

	copy := locale copy.
	copy defaultStreamEncoder: MacString encoder.
	self addLocale: copy platform: #mac.

	copy := locale copy.
	copy preferredPixelSize: nil.
	copy defaultStreamEncoder: OS2String encoder.
	self addLocale: copy platform: #os2.

	copy := locale copy.
	copy preferredPixelSize: nil.
	self addLocale: copy platform: #win32.!

installEnLocale
	"installe une Locale anglaise pour tous systemes"
	| locale copy |
	locale := self new.
	locale name: #en;
		collationPolicy: nil;
		compoundTextEncodeOrder: #(#ascii #iso1);
		currencyPolicy: nil;
		numberPolicy: nil;
		timePolicy: nil;
		cutPasteEncoding: #CompoundText;
		defaultPaperSize: 8.5 @ 11.0;
		defaultStreamEncoder: ISO8859L1String encoder;
		ignoreSerifEncodings: #();
		needsInputMethod: false;
		preferredEncodings: nil;
		preferredFontFamily: #();
		preferredPixelSize: 12.	"Taille d'un feuille de papier a lettre en inch"
	copy := locale copy.
	self addLocale: copy platform: #unix.
	copy := locale copy.
	copy defaultStreamEncoder: MacString encoder.
	self addLocale: copy platform: #mac.
	copy := locale copy.
	copy preferredPixelSize: nil.
	copy defaultStreamEncoder: OS2String encoder.
	self addLocale: copy platform: #os2.
	copy := locale copy.
	copy preferredPixelSize: nil.
	self addLocale: copy platform: #win32!

installEsLocale
	| locale copy |

	locale := self new.
	locale
		name: #es;
		collationPolicy: nil;
		compoundTextEncodeOrder: #(#ascii #iso1);
		currencyPolicy: nil;
		numberPolicy: nil;
		timePolicy: nil;
		cutPasteEncoding: #CompoundText;
		defaultPaperSize: 8.5 @ 11.0; "Letter paper size in inch"
		defaultStreamEncoder: ISO8859L1String encoder;
		ignoreSerifEncodings: #();
		needsInputMethod: false;
		preferredEncodings: nil;
		preferredFontFamily: #();
		preferredPixelSize: 12.

	copy := locale copy.
	self addLocale: copy platform: #unix.

	copy := locale copy.
	copy defaultStreamEncoder: MacString encoder.
	self addLocale: copy platform: #mac.

	copy := locale copy.
	copy preferredPixelSize: nil.
	copy defaultStreamEncoder: OS2String encoder.
	self addLocale: copy platform: #os2.

	copy := locale copy.
	copy preferredPixelSize: nil.
	self addLocale: copy platform: #win32.!

installFrLocale
	"Installe une locale francaise pour tous systemes
hm 99"

	| locale copy |
	locale := self new.
	locale name: #fr;
		collationPolicy: nil;
		compoundTextEncodeOrder: #(#ascii #iso1);
		currencyPolicy: nil;
		numberPolicy: nil;
		timePolicy: nil;
		cutPasteEncoding: #CompoundText;
		defaultPaperSize: 8.5 @ 11.0;
		defaultStreamEncoder: ISO8859L1String encoder;
		ignoreSerifEncodings: #();
		needsInputMethod: false;
		preferredEncodings: nil;
		preferredFontFamily: #();
		preferredPixelSize: 12.	"Taille d'un feuille de papier a lettre en inch"
	copy := locale copy.
	self addLocale: copy platform: #unix.
	copy := locale copy.
	copy defaultStreamEncoder: MacString encoder.
	self addLocale: copy platform: #mac.
	copy := locale copy.
	copy preferredPixelSize: nil.
	copy defaultStreamEncoder: OS2String encoder.
	self addLocale: copy platform: #os2.
	copy := locale copy.
	copy preferredPixelSize: nil.
	self addLocale: copy platform: #win32!

installUSLocale
	| locale copy |

	locale := self new.
	locale
		collationPolicy: (StringCollationPolicy newFor: #englishCollate:to:);
		compoundTextEncodeOrder: #(#ascii #iso1);
		currencyPolicy: (NumberPrintPolicy newFor: #usCurrency);
		numberPolicy: (NumberPrintPolicy newFor: #us);
		timePolicy: (TimestampPrintPolicy newFor: #us);
		cutPasteEncoding: #CompoundText;
		defaultPaperSize: 8.5 @ 11.0; "Letter paper size in inch"
		ignoreSerifEncodings: #();
		needsInputMethod: false;
		preferredEncodings: nil;
		preferredFontFamily: #();
		preferredPixelSize: 12.

	copy := locale copy.
	copy name: #'en_US.HPRoman8'.
	copy defaultStreamEncoder:
			(StreamEncoder lookupEncoderDirectory: #HP_Roman8).
	self addLocale: copy platform: #unix.

	copy := locale copy.
	copy name: #'en_US.ISO8859-1'.
	copy defaultStreamEncoder: ISO8859L1String encoder.
	self addLocale: copy platform: #unix.

	copy := locale copy.
	copy name: #'en_US.MacRoman'.
	copy defaultStreamEncoder: MacString encoder.
	self addLocale: copy platform: #mac.

	copy := locale copy.
	copy name: #'en_US.CP850'.
	copy preferredPixelSize: nil.
	copy defaultStreamEncoder: OS2String encoder.
	self addLocale: copy platform: #os2.

	copy := locale copy.
	copy name: #'en_US.CP1252'.
	copy preferredPixelSize: nil.
	copy defaultStreamEncoder: ISO8859L1String encoder.
	self addLocale: copy platform: #win32.

	LocaleMap
		at: #('* hp *' #american) put: #'en_US.HPRoman8';
		at: #('* hp *' #'american.iso88591') put: #'en_US.ISO8859-1';
		at: #('* hp *' #'en_US.roman8') put: #'en_US.HPRoman8';
		at: #('* hp *' #'en_US.iso88591') put: #'en_US.ISO8859-1';
		at: #('* sun *' #'en_US') put: #'en_US.ISO8859-1';
		at: #('* sun *' #'En_US') put: #'en_US.ISO8859-1';
		at: #('* ibm *' #'en_US') put: #'en_US.ISO8859-1';
		at: #('* sequent *' #'en_US') put: #'en_US.ISO8859-1';
		at: #('mac *' #'0-0') put: #'en_US.MacRoman';
		at: #('os2 *' #'1.850') put: #'en_US.CP850';
		at: #('win32 *' #enu) put: #'en_US.CP1252';
		at: #('win32 *' #ENU) put: #'en_US.CP1252'! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!


Core.Locale comment:
'Instances represent the defaults for a particular locale. 

Class Variables include:

	CurrentLocale <ClassName>	subclass representing the current locale
	LocaleDictionary <Dictionary>	mapping from standard locale specification to appropriate subclass
	LocaleMap <Dictionary>			mapping from array which contains operating system name and
									operating system specific locale name string to Smalltalk locale
									symbol.  You can use wildcard character * or # in operating system
									name.  The operating system name contains wildcard characters
									should match the string returned from ''ExternalInterface
									currentPlatform''

Instance Variables include:

	name <Symbol>	name of locale that instance represents
	collationPolicy <CollationPolicy> for sorting strings	
	compoundTextEncodeOrder <Array of symbols> preferred priority of encodings for compound text
	currencyPolicy <CurrencyPolicy> policy for formatting currency
	cutPasteEncoding <Symbol> encoding symbol for cut&paste
	datePolicy <String> represents date format in current locale
	defaultStreamEncoder <StreamEncoderClass>
	ignoreSerifEncodings <Array of String> encodings for which serifs are ignorable
	needsInputMethod <Boolean> true if it requires platform im support
	postScriptClass <PostScriptClass>
	preferredEncodings <Array of symbols | nil>
	preferredFontFamily <Array of Strings | nil>
	preferredPixelSize <SmallInteger | nil>
	timePolicy <TimePolicy>

Class Instance Variables include:
	dependents <Collection | nil> set of dependents that are notified when the locale changes
'!

!Core.Locale methodsFor: 'private'!

collationPolicy: aStringCollationPolicy
	collationPolicy := aStringCollationPolicy!

compoundTextEncodeOrder: anArray
	"Set the priority order of encodings in which to attempt encoding for compound text."
	compoundTextEncodeOrder := anArray!

currencyPolicy: aCurrencyFormatPolicy
	currencyPolicy := aCurrencyFormatPolicy!

cutPasteEncoding: aSymbol
	"Set the encoder symbol to be used for cut/paste with the platform"
	cutPasteEncoding := aSymbol!

defaultCollationPolicy
	"The policy to be used if the current policy is nil"
	^StringCollationPolicy newFor: #cCollate:to:!

defaultCurrencyPolicy
	"The policy to be used if the current policy is nil"

	^NumberPrintPolicy newCurrencyFor: #us!

defaultNumberPolicy
	"The policy to be used if the current policy is nil"

	^NumberPrintPolicy newFor: #us!

defaultPaperSize: size
	"Set the default paper size for printing."

	defaultPaperSize := size!

defaultStreamEncoder: aStreamEncoder
	"The default encoding of files in this locale"
	defaultStreamEncoder := aStreamEncoder!

defaultTimePolicy
	"The policy to be used if the current policy is nil"

	^TimestampPrintPolicy newFor: #us!

ignoreSerifEncodings: encodings
	ignoreSerifEncodings := encodings!

name: aSymbol
	name := aSymbol!

needsInputMethod: aBoolean
	"True iff this locale requires special input processing support"
	needsInputMethod := aBoolean!

numberPolicy: aNumberPrintPolicy

	numberPolicy := aNumberPrintPolicy!

preferredEncodings: encodingsOrNil
	"List of encodings for locale's fonts.  This is used to create a composite font of device fonts that will cover all the required character glyphs for this locale."
	preferredEncodings := encodingsOrNil!

preferredFontFamily: family
	preferredFontFamily := family!

preferredPixelSize: anInteger
	"The preferred pixelSize for the font in this locale.  The value is usually the maximum of the set of minimum sizes of each of the required fonts such that each font looks reasonable."
	preferredPixelSize := anInteger!

timePolicy: aTimePolicy
	timePolicy := aTimePolicy! !

!Core.Locale methodsFor: 'accessing'!

collationPolicy
	"This is cached only for installation purposes."
	^collationPolicy notNil
		ifTrue: [collationPolicy]
		ifFalse: [collationPolicy := self defaultCollationPolicy]!

compoundTextEncodeOrder
	^compoundTextEncodeOrder!

currencyPolicy
	"This is cached only for installation purposes."
	^currencyPolicy notNil
		ifTrue: [currencyPolicy]
		ifFalse: [currencyPolicy := self defaultCurrencyPolicy]!

cutPasteEncoding
	^cutPasteEncoding!

defaultEncoding

	^defaultStreamEncoder streamEncodingType!

defaultPaperSize
	"Answer the default paper size for printing."

	^defaultPaperSize!

defaultStreamEncoder
	^defaultStreamEncoder!

ignoreSerifEncodings
	^ignoreSerifEncodings!

languageAndTerritory
	"Answer an identifier for the language-plus-territory
	associated with this locale. This will be of the form
	'en_US', where the information before the _ defines
	the language, and the information following the _
	defines the territorial variant of the language."

	| nm |
	nm := self name asString.
	^((nm includes: $.) and: [nm includes: $_])
		ifTrue: [nm copyUpTo: $.]
		ifFalse: [nm]!

languageID
	"Answer an identifier for the language associated with this locale."

	| nm |
	nm := self name asString.
	^((nm includes: $.) and: [nm includes: $_])
		ifTrue: [nm copyUpTo: $_]
		ifFalse: [nm]!

name
	^name!

needsInputMethod
	"Predicate, returns true iff this locale requires input processing support"
	^needsInputMethod!

numberPolicy
	"This is cached only for installation purposes."
	^numberPolicy notNil
		ifTrue: [numberPolicy]
		ifFalse: [numberPolicy := self defaultNumberPolicy]!

preferredEncodings
	^preferredEncodings!

preferredFontFamily
	^preferredFontFamily!

preferredPixelSize
	"MS Windows locales select the font size at startup time.  Those locales tend to set the preferredPixelSize to nil."
	^preferredPixelSize notNil
		ifTrue: [preferredPixelSize]
		ifFalse: [Screen default defaultFontPolicy preferredFontSize]!

preferredPixelSizeFor: aFontPolicy
	"MS Windows locales select the font size at startup time.  Those locales tend to set the preferredPixelSize to nil."

	^preferredPixelSize notNil
		ifTrue: [preferredPixelSize]
		ifFalse: [aFontPolicy preferredFontSize]!

timePolicy
	"This is cached only for installation purposes"
	^timePolicy notNil
		ifTrue: [timePolicy]
		ifFalse: [timePolicy := self defaultTimePolicy]! !

!Core.Locale methodsFor: 'time and number utilities'!

printAsCurrency: aNumberThing 
	"Generate a printString for aNumberThing. This can be a Number or 
	nil where the nil is to be treated as a null representative of Number."

	^self currencyPolicy
		print: aNumberThing
		policy: self currencyPolicy policy!

printAsNumber: aNumberThing 
	"Generate a printString for aNumberThing. This can be a Number or 
	nil where the nil is to be treated as a null representative of Number."

	^self numberPolicy
		print: aNumberThing
		policy: self numberPolicy policy!

printAsTime: aTimeThing policyNamed: policyName
	"Generate a printString for aTimeThing. This can be a Time,
	Date, Timestamp, or nil where the nil is to be treated as a
	null representative of one of the above. The policy name
	is a Symbol, describing the format to be used. All Locales
	should support #long, #short, and #editing--#editing is
	a date format similar to #short, but it has differences that
	make it easier to parse unambiguously, such as using four
	digits to represent the year instead of two."

	^self timePolicy
		print: aTimeThing
		policy: (self timePolicy policyNamed: policyName)!

readDateFrom: aStream
	"Read a Date from the stream."

	^self timePolicy reader readDateFrom: aStream!

readNumberFrom: aStream type: typeClass
	"Read a Number from the stream."

	^self numberPolicy reader readFrom: aStream type: typeClass!

readTimeFrom: aStream
	"Read a Time from the stream."

	^self timePolicy reader readTimeFrom: aStream!

readTimestampFrom: aStream
	"Read a Timestamp from the stream."

	^self timePolicy reader readTimestampFrom: aStream! !


#{Core.Locale} initialize!

